"""
Copyright (C) 2019 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi.object_implem import Object


class SoftDollarTier(Object):
    def __init__(self, name="", val="", displayName=""):
        self.name = name
        self.val = val
        self.displayName = displayName

    def __str__(self):
        return f"Name: {self.name}, Value: {self.val}, DisplayName: {self.displayName}"



"""
Copyright (C) 2019 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""


from ibapi.object_implem import Object
from ibapi.const import UNSET_INTEGER, UNSET_DOUBLE


class ScanData(Object):
    def __init__(
        self,
        contract=None,
        rank=0,
        distance="",
        benchmark="",
        projection="",
        legsStr="",
    ):
        self.contract = contract
        self.rank = rank
        self.distance = distance
        self.benchmark = benchmark
        self.projection = projection
        self.legsStr = legsStr

    def __str__(self):
        return (
            "Rank: %d, Symbol: %s, SecType: %s, Currency: %s, Distance: %s, Benchmark: %s, Projection: %s, Legs String: %s"
            % (
                self.rank,
                self.contract.symbol,
                self.contract.secType,
                self.contract.currency,
                self.distance,
                self.benchmark,
                self.projection,
                self.legsStr,
            )
        )


NO_ROW_NUMBER_SPECIFIED = -1


class ScannerSubscription(Object):
    def __init__(self):
        self.numberOfRows = NO_ROW_NUMBER_SPECIFIED
        self.instrument = ""
        self.locationCode = ""
        self.scanCode = ""
        self.abovePrice = UNSET_DOUBLE
        self.belowPrice = UNSET_DOUBLE
        self.aboveVolume = UNSET_INTEGER
        self.marketCapAbove = UNSET_DOUBLE
        self.marketCapBelow = UNSET_DOUBLE
        self.moodyRatingAbove = ""
        self.moodyRatingBelow = ""
        self.spRatingAbove = ""
        self.spRatingBelow = ""
        self.maturityDateAbove = ""
        self.maturityDateBelow = ""
        self.couponRateAbove = UNSET_DOUBLE
        self.couponRateBelow = UNSET_DOUBLE
        self.excludeConvertible = False
        self.averageOptionVolumeAbove = UNSET_INTEGER
        self.scannerSettingPairs = ""
        self.stockTypeFilter = ""

    def __str__(self):
        s = "Instrument: %s, LocationCode: %s, ScanCode: %s" % (
            self.instrument,
            self.locationCode,
            self.scanCode,
        )
        return s



"""
Copyright (C) 2019 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi import comm
from ibapi.const import UNSET_DOUBLE
from ibapi.object_implem import Object
from ibapi.enum_implem import Enum
from ibapi.utils import decode


# TODO: add support for Rebate, P/L, ShortableShares conditions


class OrderCondition(Object):
    Price = 1
    Time = 3
    Margin = 4
    Execution = 5
    Volume = 6
    PercentChange = 7

    def __init__(self, condType):
        self.condType = condType
        self.isConjunctionConnection = True

    def type(self):
        return self.condType

    def And(self):
        self.isConjunctionConnection = True
        return self

    def Or(self):
        self.isConjunctionConnection = False
        return self

    def decode(self, fields):
        connector = decode(str, fields)
        self.isConjunctionConnection = connector == "a"

    def make_fields(self):
        return [comm.make_field("a" if self.isConjunctionConnection else "o")]

    def __str__(self):
        return "<AND>" if self.isConjunctionConnection else "<OR>"


class ExecutionCondition(OrderCondition):
    def __init__(self, secType=None, exch=None, symbol=None):
        OrderCondition.__init__(self, OrderCondition.Execution)
        self.secType = secType
        self.exchange = exch
        self.symbol = symbol

    def decode(self, fields):
        OrderCondition.decode(self, fields)
        self.secType = decode(str, fields)
        self.exchange = decode(str, fields)
        self.symbol = decode(str, fields)

    def make_fields(self):
        flds = OrderCondition.make_fields(self) + [
            comm.make_field(self.secType),
            comm.make_field(self.exchange),
            comm.make_field(self.symbol),
        ]
        return flds

    def __str__(self):
        return (
            "trade occurs for "
            + self.symbol
            + " symbol on "
            + self.exchange
            + " exchange for "
            + self.secType
            + " security type"
        )


class OperatorCondition(OrderCondition):
    def __init__(self, condType=None, isMore=None):
        OrderCondition.__init__(self, condType)
        self.isMore = isMore

    def valueToString(self) -> str:
        raise NotImplementedError("abstractmethod!")

    def setValueFromString(self, text: str) -> None:
        raise NotImplementedError("abstractmethod!")

    def decode(self, fields):
        OrderCondition.decode(self, fields)
        self.isMore = decode(bool, fields)
        text = decode(str, fields)
        self.setValueFromString(text)

    def make_fields(self):
        return OrderCondition.make_fields(self) + [
            comm.make_field(self.isMore),
            comm.make_field(self.valueToString()),
        ]

    def __str__(self):
        sb = ">= " if self.isMore else "<= "
        return f" {sb} {self.valueToString()}"


class MarginCondition(OperatorCondition):
    def __init__(self, isMore=None, percent=None):
        OperatorCondition.__init__(self, OrderCondition.Margin, isMore)
        self.percent = percent

    def decode(self, fields):
        OperatorCondition.decode(self, fields)

    def make_fields(self):
        flds = OperatorCondition.make_fields(self)
        return flds

    def valueToString(self) -> str:
        return str(self.percent)

    def setValueFromString(self, text: str) -> None:
        self.percent = float(text)

    def __str__(self):
        return f"the margin cushion percent {OperatorCondition.__str__(self)} "


class ContractCondition(OperatorCondition):
    def __init__(self, condType=None, conId=None, exch=None, isMore=None):
        OperatorCondition.__init__(self, condType, isMore)
        self.conId = conId
        self.exchange = exch

    def decode(self, fields):
        OperatorCondition.decode(self, fields)
        self.conId = decode(int, fields)
        self.exchange = decode(str, fields)

    def make_fields(self):
        return OperatorCondition.make_fields(self) + [
            comm.make_field(self.conId),
            comm.make_field(self.exchange),
        ]

    def valueToString(self) -> str:
        # todo
        pass

    def setValueFromString(self, text: str) -> None:
        # todo
        pass

    def __str__(self):
        return f"{self.conId} on {self.exchange} is {OperatorCondition.__str__(self)} "


class TimeCondition(OperatorCondition):
    def __init__(self, isMore=None, time=None):
        OperatorCondition.__init__(self, OrderCondition.Time, isMore)
        self.time = time

    def decode(self, fields):
        OperatorCondition.decode(self, fields)

    def make_fields(self):
        flds = OperatorCondition.make_fields(self)
        return flds

    def valueToString(self) -> str:
        return self.time

    def setValueFromString(self, text: str) -> None:
        self.time = text

    def __str__(self):
        return f"time is {OperatorCondition.__str__(self)} "


class PriceCondition(ContractCondition):
    TriggerMethodEnum = Enum(
        "Default",  # = 0,
        "DoubleBidAsk",  # = 1,
        "Last",  # = 2,
        "DoubleLast",  # = 3,
        "BidAsk",  # = 4,
        "N/A1",
        "N/A2",
        "LastBidAsk",  # = 7,
        "MidPoint",
    )  # = 8

    def __init__(
        self, triggerMethod=None, conId=None, exch=None, isMore=None, price=None
    ):
        ContractCondition.__init__(self, OrderCondition.Price, conId, exch, isMore)
        self.price = price
        self.triggerMethod = triggerMethod

    def decode(self, fields):
        ContractCondition.decode(self, fields)
        self.triggerMethod = decode(int, fields)

    def make_fields(self):
        flds = ContractCondition.make_fields(self) + [
            comm.make_field(self.triggerMethod),
        ]
        return flds

    def valueToString(self) -> str:
        return str(self.price)

    def setValueFromString(self, text: str) -> None:
        self.price = float(text)

    @property
    def __str__(self):
        return (
            f"{PriceCondition.TriggerMethodEnum.toStr(self.triggerMethod)} "
            f"price of {ContractCondition.__str__(self)} "
        )


class PercentChangeCondition(ContractCondition):
    def __init__(self, conId=None, exch=None, isMore=None, changePercent=UNSET_DOUBLE):
        ContractCondition.__init__(
            self, OrderCondition.PercentChange, conId, exch, isMore
        )
        self.changePercent = changePercent

    def decode(self, fields):
        ContractCondition.decode(self, fields)

    def make_fields(self):
        flds = ContractCondition.make_fields(self)
        return flds

    def valueToString(self) -> str:
        return str(self.changePercent)

    def setValueFromString(self, text: str) -> None:
        self.changePercent = float(text)

    def __str__(self):
        return f"percent change of {ContractCondition.__str__(self)} "


class VolumeCondition(ContractCondition):
    def __init__(self, conId=None, exch=None, isMore=None, volume=None):
        ContractCondition.__init__(self, OrderCondition.Volume, conId, exch, isMore)
        self.volume = volume

    def decode(self, fields):
        ContractCondition.decode(self, fields)

    def make_fields(self):
        flds = ContractCondition.make_fields(self)
        return flds

    def valueToString(self) -> str:
        return str(self.volume)

    def setValueFromString(self, text: str) -> None:
        self.volume = int(text)

    def __str__(self):
        return f"volume of {ContractCondition.__str__(self)} "


def Create(condType):
    cond = None

    if OrderCondition.Execution == condType:
        cond = ExecutionCondition()
    elif OrderCondition.Margin == condType:
        cond = MarginCondition()
    elif OrderCondition.PercentChange == condType:
        cond = PercentChangeCondition()
    elif OrderCondition.Price == condType:
        cond = PriceCondition()
    elif OrderCondition.Time == condType:
        cond = TimeCondition()
    elif OrderCondition.Volume == condType:
        cond = VolumeCondition()

    return cond



"""
Copyright (C) 2023 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""


from ibapi.const import UNSET_INTEGER, UNSET_DECIMAL
from ibapi.enum_implem import Enum
from ibapi.object_implem import Object
from ibapi.utils import floatMaxString, decimalMaxString, intMaxString

TickerId = int
OrderId = int
TagValueList = list

FaDataType = int
FaDataTypeEnum = Enum("N/A", "GROUPS", "N/A", "ALIASES")

MarketDataType = int
MarketDataTypeEnum = Enum("N/A", "REALTIME", "FROZEN", "DELAYED", "DELAYED_FROZEN")

Liquidities = int
LiquiditiesEnum = Enum("None", "Added", "Remove", "RoudedOut")

SetOfString = set
SetOfFloat = set
ListOfOrder = list
ListOfFamilyCode = list
ListOfContractDescription = list
ListOfDepthExchanges = list
ListOfNewsProviders = list
SmartComponentMap = dict
HistogramDataList = list
ListOfPriceIncrements = list
ListOfHistoricalTick = list
ListOfHistoricalTickBidAsk = list
ListOfHistoricalTickLast = list
ListOfHistoricalSessions = list


class BarData(Object):
    def __init__(self):
        self.date = ""
        self.open = 0.0
        self.high = 0.0
        self.low = 0.0
        self.close = 0.0
        self.volume = UNSET_DECIMAL
        self.wap = UNSET_DECIMAL
        self.barCount = 0

    def __str__(self):
        return (
            f"Date: {self.date}, "
            f"Open: {floatMaxString(self.open)}, "
            f"High: {floatMaxString(self.high)}, "
            f"Low: {floatMaxString(self.low)}, "
            f"Close: {floatMaxString(self.close)}, "
            f"Volume: {decimalMaxString(self.volume)}, "
            f"WAP: {decimalMaxString(self.wap)}, "
            f"BarCount: {intMaxString(self.barCount)}"
        )


class RealTimeBar(Object):
    def __init__(
        self,
        time=0,
        endTime=-1,
        open_=0.0,
        high=0.0,
        low=0.0,
        close=0.0,
        volume=UNSET_DECIMAL,
        wap=UNSET_DECIMAL,
        count=0,
    ):
        self.time = time
        self.endTime = endTime
        self.open_ = open_
        self.high = high
        self.low = low
        self.close = close
        self.volume = volume
        self.wap = wap
        self.count = count

    def __str__(self):
        return (
            "Time: %s, Open: %s, High: %s, Low: %s, Close: %s, Volume: %s, WAP: %s, Count: %s"
            % (
                intMaxString(self.time),
                floatMaxString(self.open_),
                floatMaxString(self.high),
                floatMaxString(self.low),
                floatMaxString(self.close),
                decimalMaxString(self.volume),
                decimalMaxString(self.wap),
                intMaxString(self.count),
            )
        )


class HistogramData(Object):
    def __init__(self):
        self.price = 0.0
        self.size = UNSET_DECIMAL

    def __str__(self):
        return "Price: %s, Size: %s" % (
            floatMaxString(self.price),
            decimalMaxString(self.size),
        )


class NewsProvider(Object):
    def __init__(self):
        self.code = ""
        self.name = ""

    def __str__(self):
        return f"Code: {self.code}, Name: {self.name}"


class DepthMktDataDescription(Object):
    def __init__(self):
        self.exchange = ""
        self.secType = ""
        self.listingExch = ""
        self.serviceDataType = ""
        self.aggGroup = UNSET_INTEGER

    def __str__(self):
        if self.aggGroup != UNSET_INTEGER:
            aggGroup = self.aggGroup
        else:
            aggGroup = ""
        return (
            "Exchange: %s, SecType: %s, ListingExchange: %s, ServiceDataType: %s, AggGroup: %s, "
            % (
                self.exchange,
                self.secType,
                self.listingExch,
                self.serviceDataType,
                intMaxString(aggGroup),
            )
        )


class SmartComponent(Object):
    def __init__(self):
        self.bitNumber = 0
        self.exchange = ""
        self.exchangeLetter = ""

    def __str__(self):
        return "BitNumber: %d, Exchange: %s, ExchangeLetter: %s" % (
            self.bitNumber,
            self.exchange,
            self.exchangeLetter,
        )


class TickAttrib(Object):
    def __init__(self):
        self.canAutoExecute = False
        self.pastLimit = False
        self.preOpen = False

    def __str__(self):
        return "CanAutoExecute: %d, PastLimit: %d, PreOpen: %d" % (
            self.canAutoExecute,
            self.pastLimit,
            self.preOpen,
        )


class TickAttribBidAsk(Object):
    def __init__(self):
        self.bidPastLow = False
        self.askPastHigh = False

    def __str__(self):
        return "BidPastLow: %d, AskPastHigh: %d" % (self.bidPastLow, self.askPastHigh)


class TickAttribLast(Object):
    def __init__(self):
        self.pastLimit = False
        self.unreported = False

    def __str__(self):
        return "PastLimit: %d, Unreported: %d" % (self.pastLimit, self.unreported)


class FamilyCode(Object):
    def __init__(self):
        self.accountID = ""
        self.familyCodeStr = ""

    def __str__(self):
        return f"AccountId: {self.accountID}, FamilyCodeStr: {self.familyCodeStr}"


class PriceIncrement(Object):
    def __init__(self):
        self.lowEdge = 0.0
        self.increment = 0.0

    def __str__(self):
        return "LowEdge: %s, Increment: %s" % (
            floatMaxString(self.lowEdge),
            floatMaxString(self.increment),
        )


class HistoricalTick(Object):
    def __init__(self):
        self.time = 0
        self.price = 0.0
        self.size = UNSET_DECIMAL

    def __str__(self):
        return "Time: %s, Price: %s, Size: %s" % (
            intMaxString(self.time),
            floatMaxString(self.price),
            decimalMaxString(self.size),
        )


class HistoricalTickBidAsk(Object):
    def __init__(self):
        self.time = 0
        self.tickAttribBidAsk = TickAttribBidAsk()
        self.priceBid = 0.0
        self.priceAsk = 0.0
        self.sizeBid = UNSET_DECIMAL
        self.sizeAsk = UNSET_DECIMAL

    def __str__(self):
        return (
            f"Time: {intMaxString(self.time)}, "
            f"TickAttriBidAsk: {self.tickAttribBidAsk}, "
            f"PriceBid: {floatMaxString(self.priceBid)}, "
            f"PriceAsk: {floatMaxString(self.priceAsk)}, "
            f"SizeBid: {decimalMaxString(self.sizeBid)}, "
            f"SizeAsk: {decimalMaxString(self.sizeAsk)}"
        )


class HistoricalTickLast(Object):
    def __init__(self):
        self.time = 0
        self.tickAttribLast = TickAttribLast()
        self.price = 0.0
        self.size = UNSET_DECIMAL
        self.exchange = ""
        self.specialConditions = ""

    def __str__(self):
        return (
            f"Time: {intMaxString(self.time)}, "
            f"TickAttribLast: {self.tickAttribLast}, "
            f"Price: {floatMaxString(self.price)}, "
            f"Size: {decimalMaxString(self.size)}, "
            f"Exchange: {self.exchange}, "
            f"SpecialConditions: {self.specialConditions}"
        )


class HistoricalSession(Object):
    def __init__(self):
        self.startDateTime = ""
        self.endDateTime = ""
        self.refDate = ""

    def __str__(self):
        return "Start: %s, End: %s, Ref Date: %s" % (
            self.startDateTime,
            self.endDateTime,
            self.refDate,
        )


class WshEventData(Object):
    def __init__(self):
        self.conId = UNSET_INTEGER
        self.filter = ""
        self.fillWatchlist = False
        self.fillPortfolio = False
        self.fillCompetitors = False
        self.startDate = ""
        self.endDate = ""
        self.totalLimit = UNSET_INTEGER

    def __str__(self):
        return (
            f"WshEventData. ConId: {intMaxString(self.conId)}, "
            f"Filter: {self.filter}, "
            f"Fill Watchlist: {self.fillWatchlist:d}, "
            f"Fill Portfolio: {self.fillPortfolio:d}, "
            f"Fill Competitors: {self.fillCompetitors:d}"
        )



"""
Copyright (C) 2019 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi.enum_implem import Enum

# TickType
TickType = int
TickTypeEnum = Enum(
    "BID_SIZE",
    "BID",
    "ASK",
    "ASK_SIZE",
    "LAST",
    "LAST_SIZE",
    "HIGH",
    "LOW",
    "VOLUME",
    "CLOSE",
    "BID_OPTION_COMPUTATION",
    "ASK_OPTION_COMPUTATION",
    "LAST_OPTION_COMPUTATION",
    "MODEL_OPTION",
    "OPEN",
    "LOW_13_WEEK",
    "HIGH_13_WEEK",
    "LOW_26_WEEK",
    "HIGH_26_WEEK",
    "LOW_52_WEEK",
    "HIGH_52_WEEK",
    "AVG_VOLUME",
    "OPEN_INTEREST",
    "OPTION_HISTORICAL_VOL",
    "OPTION_IMPLIED_VOL",
    "OPTION_BID_EXCH",
    "OPTION_ASK_EXCH",
    "OPTION_CALL_OPEN_INTEREST",
    "OPTION_PUT_OPEN_INTEREST",
    "OPTION_CALL_VOLUME",
    "OPTION_PUT_VOLUME",
    "INDEX_FUTURE_PREMIUM",
    "BID_EXCH",
    "ASK_EXCH",
    "AUCTION_VOLUME",
    "AUCTION_PRICE",
    "AUCTION_IMBALANCE",
    "MARK_PRICE",
    "BID_EFP_COMPUTATION",
    "ASK_EFP_COMPUTATION",
    "LAST_EFP_COMPUTATION",
    "OPEN_EFP_COMPUTATION",
    "HIGH_EFP_COMPUTATION",
    "LOW_EFP_COMPUTATION",
    "CLOSE_EFP_COMPUTATION",
    "LAST_TIMESTAMP",
    "SHORTABLE",
    "FUNDAMENTAL_RATIOS",
    "RT_VOLUME",
    "HALTED",
    "BID_YIELD",
    "ASK_YIELD",
    "LAST_YIELD",
    "CUST_OPTION_COMPUTATION",
    "TRADE_COUNT",
    "TRADE_RATE",
    "VOLUME_RATE",
    "LAST_RTH_TRADE",
    "RT_HISTORICAL_VOL",
    "IB_DIVIDENDS",
    "BOND_FACTOR_MULTIPLIER",
    "REGULATORY_IMBALANCE",
    "NEWS_TICK",
    "SHORT_TERM_VOLUME_3_MIN",
    "SHORT_TERM_VOLUME_5_MIN",
    "SHORT_TERM_VOLUME_10_MIN",
    "DELAYED_BID",
    "DELAYED_ASK",
    "DELAYED_LAST",
    "DELAYED_BID_SIZE",
    "DELAYED_ASK_SIZE",
    "DELAYED_LAST_SIZE",
    "DELAYED_HIGH",
    "DELAYED_LOW",
    "DELAYED_VOLUME",
    "DELAYED_CLOSE",
    "DELAYED_OPEN",
    "RT_TRD_VOLUME",
    "CREDITMAN_MARK_PRICE",
    "CREDITMAN_SLOW_MARK_PRICE",
    "DELAYED_BID_OPTION",
    "DELAYED_ASK_OPTION",
    "DELAYED_LAST_OPTION",
    "DELAYED_MODEL_OPTION",
    "LAST_EXCH",
    "LAST_REG_TIME",
    "FUTURES_OPEN_INTEREST",
    "AVG_OPT_VOLUME",
    "DELAYED_LAST_TIMESTAMP",
    "SHORTABLE_SHARES",
    "DELAYED_HALTED",
    "REUTERS_2_MUTUAL_FUNDS",
    "ETF_NAV_CLOSE",
    "ETF_NAV_PRIOR_CLOSE",
    "ETF_NAV_BID",
    "ETF_NAV_ASK",
    "ETF_NAV_LAST",
    "ETF_FROZEN_NAV_LAST",
    "ETF_NAV_HIGH",
    "ETF_NAV_LOW",
    "SOCIAL_MARKET_ANALYTICS",
    "ESTIMATED_IPO_MIDPOINT",
    "FINAL_IPO_LAST",
    "DELAYED_YIELD_BID",
    "DELAYED_YIELD_ASK",
    "NOT_SET",
)



"""
Copyright (C) 2019 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi.const import UNSET_DOUBLE


class OrderState:
    def __init__(self):
        self.status = ""

        self.initMarginBefore = ""
        self.maintMarginBefore = ""
        self.equityWithLoanBefore = ""
        self.initMarginChange = ""
        self.maintMarginChange = ""
        self.equityWithLoanChange = ""
        self.initMarginAfter = ""
        self.maintMarginAfter = ""
        self.equityWithLoanAfter = ""

        self.commission = UNSET_DOUBLE  # type: float
        self.minCommission = UNSET_DOUBLE  # type: float
        self.maxCommission = UNSET_DOUBLE  # type: float
        self.commissionCurrency = ""
        self.warningText = ""
        self.completedTime = ""
        self.completedStatus = ""



"""
Copyright (C) 2023 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""


from ibapi.object_implem import Object
from ibapi.const import UNSET_DECIMAL
from ibapi.utils import decimalMaxString
from ibapi.utils import intMaxString
from ibapi.utils import floatMaxString


class Execution(Object):
    def __init__(self):
        self.execId = ""
        self.time = ""
        self.acctNumber = ""
        self.exchange = ""
        self.side = ""
        self.shares = UNSET_DECIMAL
        self.price = 0.0
        self.permId = 0
        self.clientId = 0
        self.orderId = 0
        self.liquidation = 0
        self.cumQty = UNSET_DECIMAL
        self.avgPrice = 0.0
        self.orderRef = ""
        self.evRule = ""
        self.evMultiplier = 0.0
        self.modelCode = ""
        self.lastLiquidity = 0
        self.pendingPriceRevision = False

    def __str__(self):
        return (
            "ExecId: %s, Time: %s, Account: %s, Exchange: %s, Side: %s, Shares: %s, Price: %s, PermId: %s, "
            "ClientId: %s, OrderId: %s, Liquidation: %s, CumQty: %s, AvgPrice: %s, OrderRef: %s, EvRule: %s, "
            "EvMultiplier: %s, ModelCode: %s, LastLiquidity: %s, PendingPriceRevision: %s"
            % (
                self.execId,
                self.time,
                self.acctNumber,
                self.exchange,
                self.side,
                decimalMaxString(self.shares),
                floatMaxString(self.price),
                intMaxString(self.permId),
                intMaxString(self.clientId),
                intMaxString(self.orderId),
                intMaxString(self.liquidation),
                decimalMaxString(self.cumQty),
                floatMaxString(self.avgPrice),
                self.orderRef,
                self.evRule,
                floatMaxString(self.evMultiplier),
                self.modelCode,
                intMaxString(self.lastLiquidity),
                self.pendingPriceRevision,
            )
        )


class ExecutionFilter(Object):
    # Filter fields
    def __init__(self):
        self.clientId = 0
        self.acctCode = ""
        self.time = ""
        self.symbol = ""
        self.secType = ""
        self.exchange = ""
        self.side = ""



"""
Copyright (C) 2024 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

from ibapi.const import UNSET_INTEGER
from ibapi.object_implem import Object
from ibapi.utils import intMaxString

class OrderCancel(Object):
    def __init__(self):
        self.manualOrderCancelTime = ""
        self.extOperator = ""
        self.externalUserId = ""
        self.manualOrderIndicator = UNSET_INTEGER

    def __str__(self):
        s = "manualOrderCancelTime: %s, extOperator: %s, externalUserId: %s, manualOrderIndicator: %s" % (
            self.manualOrderCancelTime,
            self.extOperator,
            self.externalUserId,
            intMaxString(self.manualOrderIndicator),
        )

        return s



"""
Copyright (C) 2019 Interactive Brokers LLC. All rights reserved. This code is subject to the terms
 and conditions of the IB API Non-Commercial License or the IB API Commercial License, as applicable.
"""

"""
High level IB message info.
"""

# field types
INT = 1
STR = 2
FLT = 3


# incoming msg id's
class IN:
    TICK_PRICE = 1
    TICK_SIZE = 2
    ORDER_STATUS = 3
    ERR_MSG = 4
    OPEN_ORDER = 5
    ACCT_VALUE = 6
    PORTFOLIO_VALUE = 7
    ACCT_UPDATE_TIME = 8
    NEXT_VALID_ID = 9
    CONTRACT_DATA = 10
    EXECUTION_DATA = 11
    MARKET_DEPTH = 12
    MARKET_DEPTH_L2 = 13
    NEWS_BULLETINS = 14
    MANAGED_ACCTS = 15
    RECEIVE_FA = 16
    HISTORICAL_DATA = 17
    BOND_CONTRACT_DATA = 18
    SCANNER_PARAMETERS = 19
    SCANNER_DATA = 20
    TICK_OPTION_COMPUTATION = 21
    TICK_GENERIC = 45
    TICK_STRING = 46
    TICK_EFP = 47
    CURRENT_TIME = 49
    REAL_TIME_BARS = 50
    FUNDAMENTAL_DATA = 51
    CONTRACT_DATA_END = 52
    OPEN_ORDER_END = 53
    ACCT_DOWNLOAD_END = 54
    EXECUTION_DATA_END = 55
    DELTA_NEUTRAL_VALIDATION = 56
    TICK_SNAPSHOT_END = 57
    MARKET_DATA_TYPE = 58
    COMMISSION_REPORT = 59
    POSITION_DATA = 61
    POSITION_END = 62
    ACCOUNT_SUMMARY = 63
    ACCOUNT_SUMMARY_END = 64
    VERIFY_MESSAGE_API = 65
    VERIFY_COMPLETED = 66
    DISPLAY_GROUP_LIST = 67
    DISPLAY_GROUP_UPDATED = 68
    VERIFY_AND_AUTH_MESSAGE_API = 69
    VERIFY_AND_AUTH_COMPLETED = 70
    POSITION_MULTI = 71
    POSITION_MULTI_END = 72
    ACCOUNT_UPDATE_MULTI = 73
    ACCOUNT_UPDATE_MULTI_END = 74
    SECURITY_DEFINITION_OPTION_PARAMETER = 75
    SECURITY_DEFINITION_OPTION_PARAMETER_END = 76
    SOFT_DOLLAR_TIERS = 77
    FAMILY_CODES = 78
    SYMBOL_SAMPLES = 79
    MKT_DEPTH_EXCHANGES = 80
    TICK_REQ_PARAMS = 81
    SMART_COMPONENTS = 82
    NEWS_ARTICLE = 83
    TICK_NEWS = 84
    NEWS_PROVIDERS = 85
    HISTORICAL_NEWS = 86
    HISTORICAL_NEWS_END = 87
    HEAD_TIMESTAMP = 88
    HISTOGRAM_DATA = 89
    HISTORICAL_DATA_UPDATE = 90
    REROUTE_MKT_DATA_REQ = 91
    REROUTE_MKT_DEPTH_REQ = 92
    MARKET_RULE = 93
    PNL = 94
    PNL_SINGLE = 95
    HISTORICAL_TICKS = 96
    HISTORICAL_TICKS_BID_ASK = 97
    HISTORICAL_TICKS_LAST = 98
    TICK_BY_TICK = 99
    ORDER_BOUND = 100
    COMPLETED_ORDER = 101
    COMPLETED_ORDERS_END = 102
    REPLACE_FA_END = 103
    WSH_META_DATA = 104
    WSH_EVENT_DATA = 105
    HISTORICAL_SCHEDULE = 106
    USER_INFO = 107


# outgoing msg id's
class OUT:
    REQ_MKT_DATA = 1
    CANCEL_MKT_DATA = 2
    PLACE_ORDER = 3
    CANCEL_ORDER = 4
    REQ_OPEN_ORDERS = 5
    REQ_ACCT_DATA = 6
    REQ_EXECUTIONS = 7
    REQ_IDS = 8
    REQ_CONTRACT_DATA = 9
    REQ_MKT_DEPTH = 10
    CANCEL_MKT_DEPTH = 11
    REQ_NEWS_BULLETINS = 12
    CANCEL_NEWS_BULLETINS = 13
    SET_SERVER_LOGLEVEL = 14
    REQ_AUTO_OPEN_ORDERS = 15
    REQ_ALL_OPEN_ORDERS = 16
    REQ_MANAGED_ACCTS = 17
    REQ_FA = 18
    REPLACE_FA = 19
    REQ_HISTORICAL_DATA = 20
    EXERCISE_OPTIONS = 21
    REQ_SCANNER_SUBSCRIPTION = 22
    CANCEL_SCANNER_SUBSCRIPTION = 23
    REQ_SCANNER_PARAMETERS = 24
    CANCEL_HISTORICAL_DATA = 25
    REQ_CURRENT_TIME = 49
    REQ_REAL_TIME_BARS = 50
    CANCEL_REAL_TIME_BARS = 51
    REQ_FUNDAMENTAL_DATA = 52
    CANCEL_FUNDAMENTAL_DATA = 53
    REQ_CALC_IMPLIED_VOLAT = 54
    REQ_CALC_OPTION_PRICE = 55
    CANCEL_CALC_IMPLIED_VOLAT = 56
    CANCEL_CALC_OPTION_PRICE = 57
    REQ_GLOBAL_CANCEL = 58
    REQ_MARKET_DATA_TYPE = 59
    REQ_POSITIONS = 61
    REQ_ACCOUNT_SUMMARY = 62
    CANCEL_ACCOUNT_SUMMARY = 63
    CANCEL_POSITIONS = 64
    VERIFY_REQUEST = 65
    VERIFY_MESSAGE = 66
    QUERY_DISPLAY_GROUPS = 67
    SUBSCRIBE_TO_GROUP_EVENTS = 68
    UPDATE_DISPLAY_GROUP = 69
    UNSUBSCRIBE_FROM_GROUP_EVENTS = 70
    START_API = 71
    VERIFY_AND_AUTH_REQUEST = 72
    VERIFY_AND_AUTH_MESSAGE = 73
    REQ_POSITIONS_MULTI = 74
    CANCEL_POSITIONS_MULTI = 75
    REQ_ACCOUNT_UPDATES_MULTI = 76
    CANCEL_ACCOUNT_UPDATES_MULTI = 77
    REQ_SEC_DEF_OPT_PARAMS = 78
    REQ_SOFT_DOLLAR_TIERS = 79
    REQ_FAMILY_CODES = 80
    REQ_MATCHING_SYMBOLS = 81
    REQ_MKT_DEPTH_EXCHANGES = 82
    REQ_SMART_COMPONENTS = 83
    REQ_NEWS_ARTICLE = 84
    REQ_NEWS_PROVIDERS = 85
    REQ_HISTORICAL_NEWS = 86
    REQ_HEAD_TIMESTAMP = 87
    REQ_HISTOGRAM_DATA = 88
    CANCEL_HISTOGRAM_DATA = 89
    CANCEL_HEAD_TIMESTAMP = 90
    REQ_MARKET_RULE = 91
    REQ_PNL = 92
    CANCEL_PNL = 93
    REQ_PNL_SINGLE = 94
    CANCEL_PNL_SINGLE = 95
    REQ_HISTORICAL_TICKS = 96
    REQ_TICK_BY_TICK_DATA = 97
    CANCEL_TICK_BY_TICK_DATA = 98
    REQ_COMPLETED_ORDERS = 99
    REQ_WSH_META_DATA = 100
    CANCEL_WSH_META_DATA = 101
    REQ_WSH_EVENT_DATA = 102
    CANCEL_WSH_EVENT_DATA = 103
    REQ_USER_INFO = 104
